@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*- #
+# frozen_string_literal: true
 
 module Rouge
   module Lexers
@@ -37,7 +38,7 @@ module Rouge
 
       state :comment do
         rule close, Comment, :pop!
-        rule /.+(?=#{close})|.+/m, Comment
+        rule /.+?(?=#{close})|.+/m, Comment
       end
 
       state :ruby do
